@@ -7,7 +7,7 @@ use warnings;
 use Carp         'confess';
 use Scalar::Util 'blessed', 'weaken';
 
-our $VERSION   = '0.78';
+our $VERSION   = '0.79';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -226,86 +226,66 @@ Class::MOP::Method::Accessor - Method Meta Object for accessors
 
 =head1 DESCRIPTION
 
-This is a C<Class::MOP::Method> subclass which is used interally
-by C<Class::MOP::Attribute> to generate accessor code. It can
-handle generation of readers, writers, predicate and clearer
-methods, both as closures and as more optimized inline methods.
+This is a subclass of <Class::MOP::Method> which is used by
+C<Class::MOP::Attribute> to generate accessor code. It handles
+generation of readers, writers, predicates and clearers. For each type
+of method, it can either create a subroutine reference, or actually
+inline code by generating a string and C<eval>'ing it.
 
 =head1 METHODS
 
 =over 4
 
-=item B<new (%options)>
+=item B<< Class::MOP::Method::Accessor->new(%options) >>
 
-This creates the method based on the criteria in C<%options>,
-these options are:
+This returns a new C<Class::MOP::Method::Accessor> based on the
+C<%options> provided.
 
 =over 4
 
-=item I<attribute>
+=item * attribute
 
-This must be an instance of C<Class::MOP::Attribute> which this
-accessor is being generated for. This paramter is B<required>.
+This is the C<Class::MOP::Attribute> for which accessors are being
+generated. This option is required.
 
-=item I<accessor_type>
+=item * accessor_type
 
-This is a string from the following set; reader, writer, accessor,
-predicate or clearer. This is used to determine which type of
-method is to be generated.
+This is a string which should be one of "reader", "writer",
+"accessor", "predicate", or "clearer". This is the type of method
+being generated. This option is required.
 
-=item I<is_inline>
+=item * is_inline
 
-This is a boolean to indicate if the method should be generated
-as a closure, or as a more optimized inline version.
+This indicates whether or not the accessor should be inlined. This
+defaults to false.
+
+=item * name
+
+The method name (without a package name). This is required.
+
+=item * package_name
+
+The package name for the method. This is required.
 
 =back
 
-=item B<accessor_type>
+=item B<< $metamethod->accessor_type >>
 
-This returns the accessor type which was passed into C<new>.
+Returns the accessor type which was passed to C<new>.
 
-=item B<is_inline>
+=item B<< $metamethod->is_inline >>
 
-This returns the boolean which was passed into C<new>.
+Returns a boolean indicating whether or not the accessor is inlined.
 
-=item B<associated_attribute>
+=item B<< $metamethod->associated_attribute >>
 
-This returns the attribute instance which was passed into C<new>.
+This returns the L<Class::MOP::Attribute> object which was passed to
+C<new>.
 
-=item B<initialize_body>
+=item B<< $metamethod->body >>
 
-This will actually generate the method based on the specified
-criteria passed to the constructor.
-
-=back
-
-=head2 Method Generators
-
-These methods will generate appropriate code references for
-the various types of accessors which are supported by
-C<Class::MOP::Attribute>. The names pretty much explain it all.
-
-=over 4
-
-=item B<generate_accessor_method>
-
-=item B<generate_accessor_method_inline>
-
-=item B<generate_clearer_method>
-
-=item B<generate_clearer_method_inline>
-
-=item B<generate_predicate_method>
-
-=item B<generate_predicate_method_inline>
-
-=item B<generate_reader_method>
-
-=item B<generate_reader_method_inline>
-
-=item B<generate_writer_method>
-
-=item B<generate_writer_method_inline>
+The method itself is I<generated> when the accessor object is
+constructed.
 
 =back
 
@@ -315,7 +295,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2008 by Infinity Interactive, Inc.
+Copyright 2006-2009 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
