@@ -7,7 +7,7 @@ use warnings;
 use Carp         'confess';
 use Scalar::Util 'blessed', 'weaken', 'looks_like_number';
 
-our $VERSION   = '0.78';
+our $VERSION   = '0.79';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -102,7 +102,7 @@ sub generate_constructor_method_inline {
     $source .= "\n" . 'my $instance = ' . $self->meta_instance->inline_create_instance('$class');
     $source .= ";\n" . (join ";\n" => map {
         $self->_generate_slot_initializer($_, $close_over)
-    } 0 .. (@{$self->attributes} - 1));
+    } @{$self->attributes});
     $source .= ";\n" . 'return $instance';
     $source .= ";\n" . '}';
     warn $source if $self->options->{debug};
@@ -118,11 +118,9 @@ sub generate_constructor_method_inline {
 
 sub _generate_slot_initializer {
     my $self  = shift;
-    my $index = shift;
+    my $attr  = shift;
     my $close = shift;
 
-    my $attr = $self->attributes->[$index];
-
     my $default;
     if ($attr->has_default) {
         # NOTE:
@@ -197,58 +195,53 @@ Class::MOP::Method::Constructor - Method Meta Object for constructors
 
 =head1 DESCRIPTION
 
-This is a subclass of C<Class::MOP::Method> which deals with
-class constructors. This is used when making a class immutable
-to generate an optimized constructor.
+This is a subclass of C<Class::MOP::Method> which generates
+constructor methods.
 
 =head1 METHODS
 
 =over 4
 
-=item B<new (metaclass => $meta, options => \%options)>
+=item B<< Class::MOP::Method::Constructor->new(%options) >>
 
-=item B<options>
+This creates a new constructor object. It accepts a hash reference of
+options.
 
-This returns the options HASH which is passed into C<new>.
+=over 8
 
-=item B<associated_metaclass>
+=item * metaclass
 
-This returns the metaclass which is passed into C<new>.
+This should be a L<Class::MOP::Class> object. It is required.
 
-=item B<attributes>
+=item * name
 
-This returns the list of attributes which are associated with the
-metaclass which is passed into C<new>.
+The method name (without a package name). This is required.
 
-=item B<meta_instance>
+=item * package_name
 
-This returns the meta instance which is associated with the
-metaclass which is passed into C<new>.
+The package name for the method. This is required.
 
-=item B<is_inline>
+=item * is_inline
 
-This returns a boolean, but since constructors are very rarely
-not inlined, this always returns true for now.
-
-=item B<can_be_inlined>
-
-This method always returns true in this class. It exists so that
-subclasses (like in Moose) can override and do some sort of checking
-to determine whether or not inlining the constructor is safe.
-
-=item B<initialize_body>
-
-This creates the code reference for the constructor itself.
+This indicates whether or not the constructor should be inlined. This
+defaults to false.
 
 =back
 
-=head2 Method Generators 
+=item B<< $metamethod->is_inline >>
 
-=over 4
+Returns a boolean indicating whether or not the constructor is
+inlined.
 
-=item B<generate_constructor_method>
+=item B<< $metamethod->associated_metaclass >>
 
-=item B<generate_constructor_method_inline>
+This returns the L<Class::MOP::Class> object for the method.
+
+=item B<< $metamethod->can_be_inlined >>
+
+This method always returns true in this class. It exists so that
+subclasses (as in Moose) can do some sort of checking to determine
+whether or not inlining the constructor is safe.
 
 =back
 
@@ -258,7 +251,7 @@ Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2006-2008 by Infinity Interactive, Inc.
+Copyright 2006-2009 by Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
